import socket
from random import randint


def find_open_port(
        default_port=None,
        minimum_port=1024,
        maximum_port=65535):

    def get_new_port():
        return randint(minimum_port, maximum_port)

    port = default_port or get_new_port()
    port_count = maximum_port - minimum_port + 1
    closed_ports = set()
    while True:
        if not is_port_in_use(port):
            break
        closed_ports.add(port)
        if len(closed_ports) == port_count:
            raise OSError(
                'could not find an open port in '
                f'[{minimum_port}, {maximum_port}]')
        port = get_new_port()
    return port


def is_port_in_use(port):
    # https://stackoverflow.com/a/52872579
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        is_in_use = s.connect_ex(('127.0.0.1', int(port))) == 0
    return is_in_use



def escape_quotes_html(x):
    try:
        x = x.replace('"', '&#34;').replace("'", '&#39;')
    except AttributeError:
        pass
    return x


def escape_quotes_js(x):
    try:
        x = x.replace('"', '\\"').replace("'", "\\'")
    except AttributeError:
        pass
    return x



from fastapi.templating import Jinja2Templates


class TemplateResponseFactory(Jinja2Templates):

    def __init__(self, environment):
        'Assume nothing about the template environment'
        self.env = environment



from markdown import markdown


def get_html_from_markdown(text):
    html = markdown(text)
    clipped_html = html[3:-4]
    if '<p>' not in clipped_html and '</p>' not in clipped_html:
        html = html.removeprefix('<p>')
        html = html.removesuffix('</p>')
    html = html.replace('<p><', '<')
    html = html.replace('></p>', '>')
    return html



from os.path import abspath, dirname, join
from setuptools import find_packages, setup


ENTRY_POINTS = '''
'''
APP_CLASSIFIERS = [
    'Programming Language :: Python',
    'License :: OSI Approved :: MIT License',
]
APP_REQUIREMENTS = [
    'invisibleroads-macros-process',
]
FASTAPI_REQUIREMENTS = [
    'fastapi',
]
JINJA_REQUIREMENTS = [
    'jinja2',
]
MARKDOWN_REQUIREMENTS = [
    'markdown',
]
TEST_REQUIREMENTS = [
    'pytest',
    'pytest-cov',
]
FOLDER = dirname(abspath(__file__))
DESCRIPTION = '\n\n'.join(open(join(FOLDER, x)).read().strip() for x in [
    'README.md', 'CHANGES.md'])


setup(
    name='invisibleroads-macros-web',
    version='0.2.1',
    description='Shortcut functions for web operations',
    long_description=DESCRIPTION,
    long_description_content_type='text/markdown',
    classifiers=APP_CLASSIFIERS,
    author='Roy Hyunjin Han',
    author_email='rhh@crosscompute.com',
    url=(
        'https://github.com/invisibleroads/'
        'invisibleroads-macros-web'),
    keywords='invisibleroads',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=APP_REQUIREMENTS,
    extras_require={
        'fastapi': FASTAPI_REQUIREMENTS,
        'jinja': JINJA_REQUIREMENTS,
        'markdown': MARKDOWN_REQUIREMENTS,
        'test': TEST_REQUIREMENTS,
    },
    entry_points=ENTRY_POINTS)



from os.path import dirname, getmtime, join, normpath, realpath

from jinja2 import BaseLoader, Environment, TemplateNotFound, pass_context


class PathTemplateLoader(BaseLoader):

    def __init__(self, encoding='utf-8'):
        self.encoding = encoding

    def get_source(self, environment, template):
        'Support absolute template paths'
        try:
            modification_time = getmtime(template)
        except (OSError, TypeError):
            raise TemplateNotFound(template)

        def is_latest():
            try:
                return modification_time == getmtime(template)
            except OSError:
                return False

        with open(template, mode='rt', encoding=self.encoding) as f:
            text = f.read()
        return text, realpath(template), is_latest


class RelativeTemplateEnvironment(Environment):

    def join_path(self, template, parent):
        'Support relative template paths via extends, import, include'
        return normpath(join(dirname(
            parent), template)) if template else template


@pass_context
def url_for(context, name, **d):
    return context['request'].url_for(name, **d)



import webbrowser
from logging import getLogger
from time import sleep
from urllib.error import HTTPError, URLError
from urllib.request import urlopen as open_uri

from invisibleroads_macros_process import LoggableProcess


def open_browser(uri, check_interval_in_seconds=1):

    def wait_then_run():
        try:
            while True:
                try:
                    open_uri(uri)
                except HTTPError as e:
                    L.error(e)
                    return
                except URLError:
                    sleep(check_interval_in_seconds)
                else:
                    break
            webbrowser.open(uri)
        except KeyboardInterrupt:
            pass

    process = LoggableProcess(
        name='browser', target=wait_then_run, daemon=True)
    process.start()


L = getLogger(__name__)
